% CRUNCHACCEL.M will analyze the accelerometer data from the RiSE platform,
% and try to qualitatively determine the performance of each stride and the
% entire data set.  The zero-crossing point of the crank angle data will be
% used to identify separate strides within the data.

%Check for 'data' variable and load MAT file if not present
if exist('data')~=1
    [afile,apath]=uigetfile('*.mat');
    eval(['load ' apath afile])
end
%Make sure the file contains a 'data' structure
if isstruct(data)==0
    error(['NOTE: ''data'' structure is not in ''' aname '''.'])
end
% %Get the data fields
% fieldlist=fieldnames(data);
% varlist=[];
% for i=1:size(fieldlist,1)
%     %Create variable name and place data within by converting the cell name to single matrix
%     eval([cell2mat(fieldlist(i,:)) '=[data.' cell2mat(fieldlist(i,:)) ']'';'])
%     varlist=[varlist ',fieldlist(' num2str(i) ',:)'];
% end

%Make sure accel and crank data are present and extract them
if isfield(data,'Leg_toe_acceleration')==0
    error('ERROR: ''Leg_toe_acceleration'' field is not within the data structure!!!')
end
dataAccel=[data.Leg_toe_acceleration]';
if isfield(data,'Hip_crank_angle')==0
    error('ERROR: ''Hip_crank_angle'' field is not within the data structure!!!')
end
dataCrank=[data.Hip_crank_angle]';

%Determine the approximate sampling rate
datatime=[data.time]';                 %Copy the time data to another variable
gapsum=0;
for i=1:size(datatime,1)-1
    gapsum=gapsum+datatime(i+1)-datatime(i);
end
datasamp=size(datatime,1)/gapsum;    % Approximate sampling rate

%Select which leg to analyze
leg=0; fprintf('NOTE: Using leg#0 (as default)\n');
%leg=Select1Leg;

%NOTE: Figure out accelerometer data numbering scheme. Leg #3(RF) seems to
%correspond to accel data #2 (cols 4-6)??
accLegs=[4:6];
%Plot the accel data along with crank angles
%NOTE: Can do the crank stride analysis behind the scenes, no need for plotyy
figure
f1=plot(datatime,dataAccel(:,accLegs));
hold on;
%Label both axes and adjust figure
xlabel('Time [s]');
ylabel('Accel [Volts]');
title(['Accel data and for leg #' num2str(leg) ' (File:' afile ')'])
axis tight;
%Determine and plot the baseline for the accel data
%NOTE: The accel analysis will be relative to the base signal, so this is important
%NOTE: Try to take the 1st sec and last second and compare
%NOTE: Can look at crang wing angles to see if legs are moving - but this
%   doesn't mean the body(accel) is not moving
datalength=size(datatime,1);
accBaseStart=mean(dataAccel(1:round(datasamp),accLegs),1);
accBaseStop=mean(dataAccel(datalength-round(datasamp):datalength,accLegs),1);
accBaseCheck=100*abs(1-(accBaseStart./accBaseStop));
pDiff=3;    %Acceptable percentage difference between mean of 1st and last sec data
%Determine which base accel value to use on a per leg basis
for i=1:3
    if accBaseCheck(i)<pDiff
        accBase(i)=(accBaseStart(i)+accBaseStop(i))/2;
    else
        %NOTE: This mean does not seem to be working well with accel signal
        accBase(i)=mean(dataAccel(:,accLegs(1)-1+i),1);
    end
end
%Add a dotted line for the base value (hopefully with same color as accel)
curaxis=axis;
plot(curaxis(1:2),[accBase' accBase'],':','LineWidth',1)
%Determine and label the strides via the crank data
stridecount=0;
for i=1:datalength-1
    if ((dataCrank(i,leg+1)>0)&&(dataCrank(i+1,leg+1)<0))
        stridecount=stridecount+1;  %Add to the stride counter
        strides(stridecount)=i;     %Record the location of the start of the stride
        i=i+round(datasamp/10);     %Skip ahead 1/10th of a second
    end
end
for i=1:stridecount
    plot([datatime(strides(i)) datatime(strides(i))],curaxis(3:4),'k:','LineWidth',1)
end
%Shade alternate strides w/grey
for i=1:stridecount
    %Start shading with the 1st identified stride
    if(mod(i-1,2)==0)
        %Get the X-axis and Y-axis bounds
        fillxbounds(1)=datatime(strides(i));
        if (i+1<=stridecount)
            fillxbounds(2)=datatime(strides(i+1));
        else
            fillxbounds(2)=datatime(datalength);
        end
        xfill=[fillxbounds(1) fillxbounds(2) fillxbounds(2) fillxbounds(1) fillxbounds(1)];
        yfill=[curaxis(3) curaxis(3) curaxis(4) curaxis(4) curaxis(3)];
        f2=fill(xfill,yfill,[.667 .667 .667]);
        %Make the shading transparent
        alpha(f2,.3);
    end
end
%Calculate the average stride period (For possible use later)
strideperiod=mean(diff(datatime(strides)));
stridefreq=1/strideperiod;
fprintf('Avg Stride Period: %2.3f sec (%2.3fHz)\n',strideperiod, stridefreq);

%Clean up a bit
clear data params gapsum curaxis i f1 dataCrank
clear accBaseStart accBaseStop accBaseCheck pDiff
clear fillxbounds xfill yfill f2
clear strideperiod stridefreq afile apath leg

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Begin analysis here %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Analyze the accel data in chunks (by strides)
%Penalize high frequency accel data
nfft=256;                       %fft length
axcolor='bgrcmk';               % Colors for data plots
figure, fullscreen
for i=1:stridecount-1
    subplot(stridecount-1,1,i);
    legtext='';
    j=1;
    for k=accLegs
        [spec(:,j),freq(:,j)]=psd(dataAccel(strides(i):strides(i+1),k),nfft,datasamp);
        semilogy(freq(:,j),spec(:,j),axcolor(j)); hold on; grid on
        legtext=[legtext '''Accel #' num2str(k) ''', '];
        j=j+1;
    end
    title(['Stride #' num2str(i) ':Power Spectral Density of Accel Data'])
    xlabel('Freq (Hz)'), ylabel('PSD (dB)')
    eval(['legend(' legtext(1:length(legtext)-2) ')'])
end
%Penalize high amplitude accel data

%May want to filter the accel data at some point to make it easier to work
%with

%Clean up workspace some more
clear nfft axcolor legtext spec freqi j k